#!/bin/bash

# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

# debug
set -x

# Time marker for both stderr and stdout
date; date 1>&2

cloudera_config=`dirname $0`
cloudera_config=`cd "$cloudera_config/../common"; pwd`
. ${cloudera_config}/cloudera-config.sh

# attempt to find java
locate_java_home

# Load the parcel environment
source_parcel_environment

set_hive_classpath

if [ "4" = "$CDH_VERSION" ]; then
  # Disable IPv6. This is automatically done by bin/hadoop for CDH3 clients.
  export HADOOP_CLIENT_OPTS="-Djava.net.preferIPv4Stack=true $HADOOP_CLIENT_OPTS"
fi

echo "using $JAVA_HOME as JAVA_HOME"
echo "using $CDH_VERSION as CDH_VERSION"

export HADOOP_HOME=$CDH_HADOOP_HOME
export HIVE_HOME=$CDH_HIVE_HOME
export HIVE_CONF_DIR=$CONF_DIR

# Check yarn-conf first because HDFS could also generate hadoop-conf.
if [ -d $CONF_DIR/yarn-conf ]; then
  export HADOOP_CONF_DIR=$CONF_DIR/yarn-conf
  # (See CDH-10123) Hive doesn't use HADOOP_MAPRED_HOME to figure out
  # whether it should use MR1 or 2.
  # So we need to add its contents to classpath manually.
  # Also exporting it so that if it gets fixed in future, we don't have to do anything.
  export HADOOP_MAPRED_HOME=$CDH_MR2_HOME
  if [ -d $HADOOP_MAPRED_HOME ]; then
    for i in "$HADOOP_MAPRED_HOME/"*.jar; do
      AUX_CLASSPATH="${AUX_CLASSPATH}:$i"
    done
  fi
elif [ -d $CONF_DIR/hadoop-conf ]; then
  # Hive uses MR1 by default, so we don't need to set
  # HADOOP_MAPRED_HOME for MR1.
  export HADOOP_CONF_DIR=$CONF_DIR/hadoop-conf
else
  echo "No config directory found for MR."
  exit 1
fi

# core-site was generated outside because the conf file generator doesn't
# know where to put it.
if [ -f "$CONF_DIR/core-site.xml" ]; then
  cp -af "$CONF_DIR/core-site.xml" "$HADOOP_CONF_DIR"
fi

echo "using $HIVE_HOME as HIVE_HOME"
echo "using $HIVE_CONF_DIR as HIVE_CONF_DIR"
echo "using $HADOOP_HOME as HADOOP_HOME"
echo "using $HADOOP_CONF_DIR as HADOOP_CONF_DIR"

# local replacement of substitution variables.
# i.e. master doesn't know final resting place of configuration files
#   which may require absolute path (e.g. dfs_allows)
perl -pi -e "s#{{CMF_CONF_DIR}}#$CONF_DIR#g" $CONF_DIR/*.xml

JDBC_JARS="$CLOUDERA_MYSQL_CONNECTOR_JAR:$CLOUDERA_POSTGRESQL_JDBC_JAR:$CLOUDERA_ORACLE_CONNECTOR_JAR"
if [[ -z "$AUX_CLASSPATH" ]]; then
  export AUX_CLASSPATH="$JDBC_JARS"
else
  export AUX_CLASSPATH="$AUX_CLASSPATH:$JDBC_JARS"
fi


# CDH 3 hive expects the hive-default.xml to be present in conf dir
copy_hive_default "$CONF_DIR"

if [[ "$1" == webhcat ]]; then
  export HCAT_PREFIX="$CDH_HCAT_HOME"
  export WEBHCAT_CONF_DIR="$CONF_DIR"
  export HADOOP_PREFIX="$HADOOP_HOME"
  export HADOOP_LIBEXEC_DIR="$CDH_HADOOP_HOME/libexec/"
  export WEBHCAT_LOG_DIR="$HIVE_LOG_DIR"
  export WEBHCAT_LOG4J="file:$WEBHCAT_CONF_DIR/webhcat-log4j.properties"
  echo "using $HCAT_PREFIX as HCAT_PREFIX"
  echo "using $WEBHCAT_CONF_DIR as WEBHCAT_CONF_DIR"
  echo "using $HADOOP_PREFIX as HADOOP_PREFIX"
  echo "using $JAVA_HOME as JAVA_HOME"
  echo "using $HADOOP_LIBEXEC_DIR as HADOOP_LIBEXEC_DIR"
  echo "using $WEBHCAT_LOG_DIR as WEBHCAT_LOG_DIR"
  echo "using $WEBHCAT_LOG4J as WEBHCAT_LOG4J"
  WEBHCAT_SERVER_CMD="$HCAT_PREFIX/sbin/webhcat_server.sh --config \"$WEBHCAT_CONF_DIR\" foreground"

  if [ -f "$HCAT_PREFIX/share/webhcat/java-client/webhcat-java-client-0.4.0-cdh4.2.0.jar" ]; then
    # CDH4.2.0 has a buggy webhcat_server.sh that fails to kill the real webhcat server
    # trap SIGTERM to nothing, forcing supervisor to do a group SIGKILL, which will correctly clean up all processes
    trap "" SIGTERM
    $WEBHCAT_SERVER_CMD
    # this process is always killed, shouldn't exit here unless WebHCat dies abnormally
    exit $?
  else
    # CDH4.2.1+ webhcat_server.sh does not require special kill handling
    exec $WEBHCAT_SERVER_CMD
  fi

elif [[ "$1" == updatelocation ]]; then
  NAMESERVICE_NAME="$2"
  echo "Updating Hive Metastore to use Nameservice $NAMESERVICE_NAME"
  FS_ROOT_OUTPUT=`$HIVE_HOME/bin/hive --config $CONF_DIR --service metatool -listFSRoot`
  FS_ROOT_RETURN=$?
  echo "List FS Root output:"
  echo $FS_ROOT_OUTPUT
  # last line of output is the fs root
  FS_ROOT=$(echo "$FS_ROOT_OUTPUT" | tail -n 1)

  # make sure we got a legal hdfs location and not some error
  if [[ 0 == $FS_ROOT_RETURN && "$FS_ROOT" =~ ^hdfs://.* ]] ; then
    echo "Old FS Root: $FS_ROOT"
  else
    echo "ERROR: Failed while listing FS Root."
    exit 1
  fi

  # FS_ROOT=hdfs://oldhost:1234/path/to/warehouse       where 1234 is the port
  # split on / to get host and port "oldhost:1234"
  OLD_LOC_WITH_PORT=$(echo "$FS_ROOT" | awk '{split($0,array,"/")} END {print array[3]}')
  # add back "hdfs://" prefix
  OLD_LOC_WITH_PORT="hdfs://$OLD_LOC_WITH_PORT"

  # FS_ROOT=hdfs://oldhost:1234/path/to/warehouse       where 1234 is the port
  # split on : to cut out everything after the host name, getting "//oldhost"
  OLD_LOC=$(echo "$FS_ROOT" | awk '{split($0,array,":")} END {print array[2]}')
  # add back "hdfs:" prefix
  OLD_LOC="hdfs:$OLD_LOC"

  if [[ "hdfs://$NAMESERVICE_NAME" == "$OLD_LOC" ]]; then
    echo "FS Root already matches $NAMESERVICE_NAME. No Update Required"
    exit 0
  fi

  # Try passing OLD_LOC=oldhost:1234, newloc=NAMESERVICE_NAME. Note that newloc has no port.
  if $HIVE_HOME/bin/hive --config $CONF_DIR --service metatool -updateLocation "hdfs://$NAMESERVICE_NAME" "$OLD_LOC_WITH_PORT"; then
    # Success
    exit 0
  fi

  # HIVE-3665 not fixed in this build. Use old location without port.

  exec $HIVE_HOME/bin/hive --config $CONF_DIR --service metatool -updateLocation "hdfs://$NAMESERVICE_NAME" "$OLD_LOC"
elif [[ "$1" == create_metastore_tables ]]; then
  if [[ -z "$CMF_SERVER_ROOT" ]]; then
    CREATE_TABLES_CLASSPATH="/usr/share/cmf/lib/*:$JDBC_JARS"
  else
    CREATE_TABLES_CLASSPATH="$CMF_SERVER_ROOT/lib/*:$JDBC_JARS"
  fi
  exec $JAVA_HOME/bin/java -Djava.net.preferIPv4Stack=true -cp "$CREATE_TABLES_CLASSPATH" com.cloudera.cmf.service.hive.HiveMetastoreDbUtil "$CONF_DIR/metastore_db_py.properties" "$(dirname $0)/ddl" "--createTables"
fi

exec $HIVE_HOME/bin/hive --config $CONF_DIR --service "$@"
