// (c) Copyright 2013 Cloudera, Inc. All rights reserved.
define([
  'cloudera/impala/QueryResponseHandler',
  'cloudera/Util',
  'underscore'
], function(queryResponseHandler, Util, _) {

  var makeTestData = function(length) {
    var results = [];
    _.times(length, function(i) {
      results.push({
        startTime: 1348766817260 + (1000000 * i),
        endTime: 1348767817260 + (1000000 * i),
        statement: 'query ' + i
      });
    });
    return {
      queries: results
    };
  };

  describe('QueryResponseHandler', function() {
    describe('option validation', function() {
      var options;

      var getQueryResponseHandler = function() {
        return queryResponseHandler(options);
      };

      beforeEach(function() {
        options = {
          executeQuery: jasmine.createSpy('executeQuery'),
          getResults: jasmine.createSpy('getResults'),
          orderingProperty: 'catpants',
          getMaximum: jasmine.createSpy('getMaximum').andReturn('doggyhat')
        };
      });

      it('works if everything is there', function() {
        expect(getQueryResponseHandler).not.toThrow();
      });

      it('validates executeQuery is there', function() {
        options.executeQuery = null;
        expect(getQueryResponseHandler).toThrow();
      });

      it('validates getResults is there', function() {
        options.getResults = null;
        expect(getQueryResponseHandler).toThrow();
      });

      it('validates orderingProperty is there', function() {
        options.orderingProperty = null;
        expect(getQueryResponseHandler).toThrow();
      });
    });

    describe('dataSource', function() {
      var mockSearchResults, dataSource, options;

      beforeEach(function() {
        mockSearchResults = {
          clear: jasmine.createSpy('clear'),
          length: jasmine.createSpy('length').andReturn(0)
        };
        options = {
          executeQuery: jasmine.createSpy('executeSpy'),
          getResults: jasmine.createSpy('getResults'),
          orderingProperty: 'catpants',
          getMaximum: jasmine.createSpy('getMaximum').andReturn('doggyhat')
        };
        dataSource = queryResponseHandler(options);
      });

      it('clears when told to do so', function() {
        dataSource(mockSearchResults, true);
        expect(mockSearchResults.clear).wasCalled();
      });

      it('executes query with correct handlers', function() {
        // Return sentinel from makeOnRetrieveHistorySuccess.
        var callback = jasmine.createSpy('callback');
        dataSource(mockSearchResults, false, callback);
        expect(mockSearchResults.clear).wasNotCalled();
        expect(options.executeQuery).wasCalled();
        var args = options.executeQuery.mostRecentCall.args;
        var handlers = args[0];
        // Check the handlers.
        expect(_.isFunction(handlers.success)).toBeTruthy();
        expect(_.isFunction(handlers.error)).toBeTruthy();
        expect(_.isFunction(handlers.complete)).toBeTruthy();
        // Quick check on the error handler.
        handlers.error(null, null, 'catpants');
        handlers.complete();
        expect(callback).wasCalledWith('catpants');
      });
    });

    describe('createOnRetrieveHistorySuccess callback', function() {
      var mockSearchResults, mockData, options, dataSource, callback, successHandler;

      var makeSuccessHandler = function() {
        successHandler = dataSource.createOnRetrieveHistorySuccess(mockSearchResults, false);
      };

      beforeEach(function() {
        mockSearchResults = {
          length: jasmine.createSpy('length'),
          append: jasmine.createSpy('append'),
          prepend: jasmine.createSpy('prepend'),
          getHead: jasmine.createSpy('getHead'),
          getTail: jasmine.createSpy('getTail')
        };
        mockData = makeTestData(3);
        mockSearchResults.length.andReturn(0);
        options = {
          executeQuery: jasmine.createSpy('executeSpy'),
          getResults: jasmine.createSpy('getResults').andReturn(mockData.queries),
          orderingProperty: 'endTime',
          getMaximum: jasmine.createSpy('getMaximum').andReturn(999999)
        };
        dataSource = queryResponseHandler(options);
        callback = jasmine.createSpy('callback');
        makeSuccessHandler();
      });

      it('does nothing if another request has come in before it', function() {
        // Increments the internal state so that a call to the callback
        // won't apply the data.
        dataSource(mockSearchResults, false, callback);
        successHandler(mockData);
        expect(mockSearchResults.append).wasNotCalled();
      });

      it('does nothing if no data came back', function() {
        successHandler(null);
        expect(mockSearchResults.append).wasNotCalled();
      });

      it('gives the onDataReceived function a chance to do something', function() {
        options.onDataReceived = jasmine.createSpy('onDataReceived');
        successHandler('catpants');
        expect(options.onDataReceived).wasCalledWith('catpants');
      });

      it('does nothing if data has length 0', function() {
        options.getResults.andReturn([]);
        dataSource = queryResponseHandler(options);
        makeSuccessHandler();
        successHandler(mockData);
        expect(options.getResults).wasCalled();
        expect(mockSearchResults.append).wasNotCalled();
      });

      it('does nothing if result lengths do not match', function() {
        mockSearchResults.length.andReturn(5);
        successHandler(mockData);
        expect(mockSearchResults.append).wasNotCalled();
      });

      it('appends results if there were none to begin with', function() {
        var testData = makeTestData(5);
        successHandler(testData);
        expect(mockSearchResults.append).wasCalled();
        expect(mockSearchResults.getTail).wasNotCalled();
      });

      it('appends data if response from server is older than current oldest', function() {
        mockSearchResults.length.andReturn(5);
        // Set initial length.
        makeSuccessHandler();
        var testData = makeTestData(3);
        mockSearchResults.getTail.andReturn({
          endTime: testData.queries[2].endTime + 1000000
        });
        successHandler(testData);
        expect(mockSearchResults.append).wasCalled();
      });

      // This test is more about coverage than it is about testing
      // the underlying logic. So, crummy test. It's true purpose is to
      // provide a scaffold to let me know if I broke things. As such,
      // testing a lot of permutations of data getting prepended is not
      // the immediate goal.
      it('prepends data if newer and found valid prepend range', function() {
        mockSearchResults.length.andReturn(5);
        // Set initial length.
        makeSuccessHandler();
        var testData = makeTestData(3);
        mockSearchResults.getTail.andReturn({
          endTime: testData.queries[0].endTime - 50
        });
        mockSearchResults.getHead.andReturn({
          endTime: testData.queries[2].endTime + 50
        });
        spyOn(Util, 'findRange').andReturn({
          start: 0,
          end: testData.length - 1
        });
        successHandler(testData);
        expect(mockSearchResults.getTail).wasCalled();
        expect(mockSearchResults.getHead).wasCalled();
        expect(mockSearchResults.prepend).wasCalled();
      });
    });
  });
});